from dataclasses import dataclass
from typing import Iterator

from korbit.models.issue import Issue


@dataclass
class ReportCategory:
    category: str
    issues: list[Issue]

    @classmethod
    def from_json(cls, json_data):
        issues = [Issue.from_json(issue) for issue in json_data["files"]]
        return cls(
            category=json_data["category"],
            issues=issues,
        )

    def get_total_ignored_issues(self) -> int:
        return sum(len(issue.ignored_issue_files) for issue in self.issues)

    def get_total_selected_issues(self) -> int:
        return sum(len(issue.issue_files) for issue in self.issues)


@dataclass
class Report:
    scan_id: int
    report_path: str
    categories: list[ReportCategory]

    @classmethod
    def from_json(cls, json_data):
        computed_categories = {}

        for entry in json_data["issues"]:
            category = entry["files"][0]["category"]
            if category not in computed_categories:
                computed_categories[category] = []
            computed_categories[category].append(entry)

        return cls(
            scan_id=json_data["scan_id"],
            report_path=json_data["report_path"],
            categories=[
                ReportCategory.from_json({"category": category, "files": files})
                for category, files in computed_categories.items()
            ],
        )

    def categories_iterator(self) -> Iterator[tuple[str, list[Issue]]]:
        for category in self.categories:
            yield category.category, category.issues

    def get_issues_stats(self) -> tuple[int, int, int]:
        "Return the issues count, selected issues and ignored issues"
        total_issues = 0
        selected_issues = 0
        ignored_issues = 0
        for category_issues in self.categories:
            ignored_issues += category_issues.get_total_ignored_issues()
            selected_issues += category_issues.get_total_selected_issues()
        total_issues += selected_issues + ignored_issues
        return total_issues, selected_issues, ignored_issues

    def is_successful(self) -> bool:
        _, selected_issues, _ = self.get_issues_stats()
        return selected_issues == 0



import os

# API Endpoints
KORBIT_HOST = os.getenv("KORBIT_HOST", "https://mentor.korbit.ai:8000")
KORBIT_SCAN_REPORT_URL = f"{KORBIT_HOST}/database/v3/scans"
KORBIT_CODE_ANALYSIS_SERVICE = f"{KORBIT_HOST}/code-analysis/check"
KORBIT_CODE_ANALYSIS_CHECK = f"{KORBIT_CODE_ANALYSIS_SERVICE}/files"

# Local paths
KORBIT_LOCAL_FOLDER = ".korbit"
KORBIT_LOCAL_OUTPUT_LOG_FILE = f"{KORBIT_LOCAL_FOLDER}/scan.log"

# Exit codes
KORBIT_COMMAND_EXIT_CODE_UNKNOWN_ERROR = 90
KORBIT_COMMAND_EXIT_CODE_ISSUES_FOUND_WITHIN_THRESHOLD = 91
KORBIT_COMMAND_EXIT_CODE_CHECK_FAILED = 92
KORBIT_COMMAND_EXIT_CODE_AUTH_FAILED = 93



import base64
import json
import os
from pathlib import Path
from typing import Callable

import requests

from korbit.interface import INTERFACE_AUTH_MISSING_CREDENTIALS_MSG

KORBIT_CREDENTIAL_FILE = os.path.expanduser("~/.korbit/credentials")


def store_credentials(secret_id, secret_key):
    """
    Store user credentials for future usage of scan command.
    """
    os.makedirs(Path(KORBIT_CREDENTIAL_FILE).parent, exist_ok=True)
    with open(KORBIT_CREDENTIAL_FILE, "w+") as credential_file:
        json.dump({"secret_id": secret_id, "secret_key": secret_key}, credential_file)


def compute_user_token():
    with open(KORBIT_CREDENTIAL_FILE, "r") as credential_file:
        credentials = json.loads(credential_file.read())
    secret_id = os.getenv("KORBIT_SECRET_ID", credentials.get("secret_id"))
    secret_key = os.getenv("KORBIT_SECRET_KEY", credentials.get("secret_key"))
    assert secret_id, INTERFACE_AUTH_MISSING_CREDENTIALS_MSG
    assert secret_key, INTERFACE_AUTH_MISSING_CREDENTIALS_MSG
    return base64.b64encode(f"{secret_id}:{secret_key}".encode()).decode()


def authenticate_request(method: Callable[[str], requests.Response], url: str, **kwargs) -> requests.Response:
    headers = kwargs.pop("headers", {})
    if not headers.get("Authorization"):
        headers["Authorization"] = f"Basic {compute_user_token()}"
    kwargs["headers"] = headers
    response = method(url, **kwargs)
    response.raise_for_status()
    return response
